<?php // vim:ts=4:sw=4:noet
/*====================================================================== 
Copyright (C) 2006-2009, Walter Doekes
This file is part of KjwLib-php45.
 
KjwLib-php45 is free software: you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, or 
(at your option) any later version. 

KjwLib-php45 is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with KjwLib-php45.  If not, see <http://www.gnu.org/licenses/>. 
======================================================================*/

require_once(dirname(__FILE__) . '/KjwLib.php');

/**
 * Create directory. Tries to create it recursively.
 * Create mode is 2775.
 *
 * @param $path The directory to create.
 * @return Boolean specifying whether the directory exists afterwards.
 */
function kjwfs_mkdir($path) {
	if (is_dir($path))
		return true;
	if (@mkdir($path, 02775))
		return true;
	// Try to create parent(s)
	$n = strrpos($path, '/'); // DIRECTORY_SEPARATOR
	if ((int)$n === 0 || !kjwfs_mkdir(substr($path, 0, $n)))
		return false;
	// Try again
	if (@mkdir($path, 02775))
		return true;
	return false;
}

/**
 * Remove a directory and optionally all it's children.
 *
 * @param $path The directory to remove.
 * @param $filicide Boolean which tells us to remove children files as well
 *        and not just children directories.
 * @return Boolean specifying whether the directory is gone.
 */ 
function kjwfs_rmdir($path, $filicide = false) {
	$path = realpath($path);
	if (!is_dir($path))
		return false;
	$dp = opendir($path);
	$failed = false;
	while (($file = readdir($dp))) {
		if ($file != '.' && $file != '..') {
			$absfile = $path . '/' . $file;
			if (is_dir($absfile))
				$status = kjwfs_rmdir($absfile, $filicide);
			elseif ($filicide)
				$status = @unlink($absfile);
			else
				$status = false;
			if (!$status)
				$failed = true;
		}
	}
	closedir($dp);
	if ($failed)
		return false;
	return rmdir($path);
}

function kjwfs_file_put_contents($path, $data) {
	if (!kjwfs_mkdir(dirname($path)))
		return false;
	return file_put_contents($path, $data);
}
		
?>
