<?php // vim:ts=4:sw=4:noet
/*====================================================================== 
Copyright (C) 2006-2009, Walter Doekes
This file is part of KjwLib-php45.
 
KjwLib-php45 is free software: you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation, either version 3 of the License, or 
(at your option) any later version. 

KjwLib-php45 is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with KjwLib-php45.  If not, see <http://www.gnu.org/licenses/>. 
======================================================================*/

// todo, fix @@error -> translation

if (!function_exists('sybase_connect'))
	trigger_error('sybase_* functions not defined. Install php*-sybase.', E_USER_ERROR);
require_once(dirname(__FILE__) . '/KjwResultSet.php');

/** @ingroup sql
 *
 * KjwMsResultSet is an implementation for mssql of the KjwResultSet abstract class.
 */
class KjwMsResultSet extends KjwResultSet {
	var $_resource;		/**< The MsSQL result set resource. */
	var $_pointer;		/**< A pointer to keep track of the current position. */
	var $_end;			/**< The size of the result set. */

	/**
	 * Construct a KjwMsResultSet
	 *
	 * @param $resource An mssql result set resource.
	 */
	function KjwMsResultSet(&$resource) {
		parent::KjwResultSet();
		$this->_resource = &$resource;
		$this->_pointer = 0;
		$this->_end = sybase_num_rows($this->_resource);
	}

	/* [INHERITED DOCS] */
	function destroy() {
		sybase_free_result($this->_resource);
		parent::destroy();
	}

	/* [INHERITED DOCS] */
	function getNext() {
		if ($this->_pointer < $this->size()) {
			++$this->_pointer;
			return sybase_fetch_assoc($this->_resource);
		}
		return false;
	}

	/* [INHERITED DOCS] */
	function at() {
		return $this->_pointer;
	}

	/* [INHERITED DOCS] */
	function size() {
		return $this->_end;
	}

	/* [INHERITED DOCS] */
	function seekAbs($offset) {
		if ($offset >= 0 && $offset < $this->_end) {
			$this->_pointer = $offset;
			sybase_data_seek($this->_resource, $offset);
			return true;
		}
		return false;
	}
}

?>
